
/*
One important thing to keep in mind when working with signatures is that, 
normally, they will only have effect while the signed module is executed. 
When executing a signed module, the signatures will be temporarily added to the SQL token, 
but they will be lost if the module executes another one or if it terminates execution. 

This is the essence of the TSQL example shown below. 
In the example, ProcAlice appears as proc_select_t_for_alice, 
and ProcT is proc_select_t. 
*/

-- A demo for counter signatures
--
-- create test database
--
create database test_cs
go
use test_cs
-- create table t
--
create table t (c int)
insert into t values (42)
-- create a bogus user to own t
--
create user bogus without login
alter authorization on t to bogus
-- create a certificate for signing
--
create certificate cs_select_t
  encryption by password = 'SimplePwd01'
  with subject = 'Certificate used to grant SELECT on t'
  
create user ucs_select_t from certificate cs_select_t
grant select on t to ucs_select_t
-- create a principal with low privileges
--
create login alice with password = 'SimplePwd01'
create user alice
-- verify alice cannot access t
--
execute as login = 'alice'
    select * from sys.login_token
    select db_name()
    select * from sys.user_token
    select * from t
revert
-- create a procedure that directly accesses t
--
create procedure proc_select_t as
begin
    select * from sys.login_token
    select db_name()
    select * from sys.user_token
    print 'Now selecting from t...'
    select * from t
end
go
grant execute on proc_select_t to public
-- verify procedure
--
exec proc_select_t
-- verify alice cannot access t through procedure
--
execute as login = 'alice'
    exec proc_select_t
revert
-- create special procedure for accessing t
-- this will call proc_select_t but will do some extra processing
--
create procedure proc_select_t_for_alice as
begin
    select * from sys.login_token
    select db_name()
    select * from sys.user_token
    if user_id() <> user_id('alice')
    begin
        print 'Only alice can use this'
        return
    end
    exec proc_select_t
end
go
grant execute on proc_select_t_for_alice to public
-- verify procedure
--
exec proc_select_t_for_alice
-- alice still can't use the procedure yet
--
execute as login = 'alice'
    exec proc_select_t_for_alice
revert
-- Sign procedure to grant it SELECT permission
--
add signature to proc_select_t_for_alice by certificate cs_select_t with password = 'SimplePwd01'
-- retry - it still won't work, but we'll see some nice tokens
--
execute as login = 'alice'
    exec proc_select_t_for_alice
revert
-- Counter sign proc_select_t, to make this work
--
add counter signature to proc_select_t by certificate cs_select_t with password = 'SimplePwd01'
-- retry - now it finally works
-- note that calling proc_select_t directly still doesn't work
--
execute as login = 'alice'
    exec proc_select_t_for_alice
    exec proc_select_t
revert
-- cleanup
--
use master
go
drop database test_cs
drop login alice
--
-- EOD
